"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

var _integerOverflowError = _interopRequireDefault(require("./integer-overflow-error"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pushIncrementalImport = async ({
  executeQuery,
  eventsTableName,
  escapeId,
  escape
}, events, importId) => {
  try {
    const incrementalImportTableAsId = escapeId(`${eventsTableName}-incremental-import`);
    const incrementalImportTableAsString = escape(`${eventsTableName}-incremental-import`);
    await executeQuery(`BEGIN IMMEDIATE;
      SELECT ABS("CTE"."IncrementalImportFailed") FROM (
        SELECT 0 AS "IncrementalImportFailed"
      UNION ALL
        SELECT -9223372036854775808 AS "IncrementalImportFailed"
        FROM "sqlite_master"
        WHERE "type" = 'table' AND 
        "name" = ${incrementalImportTableAsString} AND
        "sql" NOT LIKE ${escape(`%-- RESOLVE INCREMENTAL-IMPORT ${escape(importId)} OWNED TABLE%`)}
      ) CTE;
      
      INSERT INTO ${incrementalImportTableAsId}(
        "timestamp", "aggregateId", "type", "payload", "threadId"
      ) VALUES ${events.map(event => `(${+event.timestamp}, ${escape(event.aggregateId)}, ${escape(event.type)}, 
      ${event.payload != null ? escape(JSON.stringify(event.payload)) : escape('null')}, ${event.timestamp % _eventstoreBase.THREAD_COUNT})`).join(',')};
    
    COMMIT;
      `);
  } catch (error) {
    try {
      await executeQuery(`ROLLBACK;`);
    } catch (e) {}

    if (error != null && ((0, _integerOverflowError.default)(error.message) || /^.*? not exists$/.test(error.message))) {
      throw new Error(`Incremental importId=${importId} does not exist`);
    } else {
      throw error;
    }
  }
};

var _default = pushIncrementalImport;
exports.default = _default;
//# sourceMappingURL=push-incremental-import.js.map